'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var REQUIRE_PATTERN = /^~\//;

function testPattern(value, pattern) {
  var type = typeof pattern === 'undefined' ? 'undefined' : _typeof(pattern);

  if (type !== 'undefined' && type !== 'string') {
    throw Error('Invalid require prefix pattern.');
  }

  var regexp = type === 'string' ? new RegExp(pattern) : REQUIRE_PATTERN;

  if (!regexp.test(value)) {
    return false;
  }

  return value.replace(regexp, '');
}

function resolveAbsolute(filename) {
  filename = _path2.default.normalize(filename);
  if (_path2.default.isAbsolute(filename)) {
    return filename;
  }

  if (process.env.PWD) {
    return _path2.default.resolve(process.env.PWD, filename);
  }

  return _path2.default.resolve(filename);
}

function makeRelative(basePath, modulePath) {
  var relativePath = _path2.default.relative(basePath, modulePath);

  if (!relativePath.startsWith('.')) {
    relativePath = './' + relativePath;
  }

  return relativePath;
}

function getOverrides(currentPath, projectPath, overrides) {
  var matches = [];

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = overrides[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var override = _step.value;

      var pathPrefix = _path2.default.join(projectPath, override);

      if (typeof override === 'string' && currentPath.startsWith(pathPrefix)) {
        matches.push(override);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return matches;
}

function getBasePath(currentPath, projectPath, options) {
  var _options$overrides = options.overrides;
  var overrides = _options$overrides === undefined ? [] : _options$overrides;
  var _options$basePath = options.basePath;
  var basePath = _options$basePath === undefined ? '' : _options$basePath;

  var matches = [].concat(_toConsumableArray(getOverrides(currentPath, projectPath, overrides)));

  return resolveAbsolute(matches.pop() || basePath);
}

function makeModulePathRelative(currentFile, node, options) {
  var requirePath = testPattern(node.value, options.pattern);

  if (requirePath) {
    var currentPath = resolveAbsolute(_path2.default.dirname(currentFile));
    var rootPath = resolveAbsolute('.');
    var basePath = getBasePath(currentPath, rootPath, options);

    node.value = makeRelative(currentPath, resolveAbsolute(_path2.default.join(basePath, requirePath)));
  }
}

exports.default = function (_ref) {
  var t = _ref.types;

  return {
    visitor: {
      CallExpression: {
        exit: function exit(_ref2, state) {
          var node = _ref2.node;

          if (!t.isIdentifier(node.callee, { name: 'require' }) && !(t.isMemberExpression(node.callee) && t.isIdentifier(node.callee.object, { name: 'require' }))) {
            return;
          }

          var currentFile = state.file.opts.filename;
          var moduleNode = node.arguments[0];

          if (t.isStringLiteral(moduleNode)) {
            makeModulePathRelative(currentFile, moduleNode, state.opts);
          }
        }
      },
      ImportDeclaration: {
        exit: function exit(_ref3, state) {
          var node = _ref3.node;

          var currentFile = state.file.opts.filename;
          var moduleNode = node.source;

          if (t.isStringLiteral(moduleNode)) {
            makeModulePathRelative(currentFile, moduleNode, state.opts);
          }
        }
      }
    }
  };
};